@@ -1,7 +1,7 @@
 # -*- mode: Perl -*-
 package MRTG_lib;
 ###################################################################
-# MRTG 2.16.4  Support library MRTG_lib.pm
+# MRTG 2.17.0  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <tobi@oetiker.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -156,6 +156,12 @@ $VERSION = 2.100016;
        'conversioncode' =>
        [sub{-r $_[0]}, sub{"Cannot read conversion code file $_[0]"}],
 
+       # Check for an environment setting for RRDCACHED_ADDRESS
+       # Steve Shipway, Sep 2010
+       'rrdcached' =>
+#       [sub{(($_[0] =~ /^unix:(\S+)/)and(-w $1))}, sub{"Currently, only UNIX domain sockets are supported for RRDCached, and must exist and be writeable."}],
+       [sub{1},sub{"Internal Error"}],
+
        # Per Router CFG
        'target[]' => 
        [sub{1}, sub{"Internal Error"}], #will test this later
@@ -463,14 +469,25 @@ sub readcfg ($$$$;$$) {
         chomp;
         my $line = $.;
         if (/^include:\s*(.*?\S)\s*$/i){
-                push @filestack, $file;
-                push @handstack, $hand;
+                my $newhandle;
+                my @nextfiles;
                 $nextfile = $1;
-                local *FH;
-                open (FH, $nextfile)
-                 || open (FH, ($cfgfile =~ m#(.+)${MRTG_lib::SL}[^${MRTG_lib::SL}]+$#)[0] . ${MRTG_lib::SL} . $nextfile)
-                 || do { die "ERROR: unable to open include file: $nextfile\n"};
-                $hand = *FH;
+                if( $nextfile =~ /\*/ ) {
+                    @nextfiles = glob( $nextfile );
+                    @nextfiles = glob( ($cfgfile =~ m#(.+)${MRTG_lib::SL}[^${MRTG_lib::SL}]+$#)[0] . ${MRTG_lib::SL} . $nextfile )
+                        if(!@nextfiles);	
+                } else { 
+                    $nextfile =  ($cfgfile =~ m#(.+)${MRTG_lib::SL}[^${MRTG_lib::SL}]+$#)[0] . ${MRTG_lib::SL} . $nextfile 
+                        if(!-r $nextfile);
+                    @nextfiles = ( $nextfile ); 
+                }
+                foreach $nextfile ( @nextfiles ) {
+                    open my $newhandle, '<', $nextfile or die "ERROR: unable to open include file: $nextfile\n";
+                    push @handstack, $hand;
+                    push @filestack, $file;
+                    $hand = $newhandle;
+                    $file = $nextfile;
+                }
                 next;
         }
 
@@ -631,6 +648,21 @@ sub readcfg ($$$$;$$) {
     }
 
     close (CFG);
+
+	# Check for an environment setting for RRDCACHED_ADDRESS
+	# Steve Shipway, Sep 2010
+	if( $ENV{RRDCACHED_ADDRESS} and not exists $cfg->{ rrdcached } ) {
+		warn("WARNING: Using environment variable RRDCACHED_ADDRESS\n");
+		$cfg->{ rrdcached } = $ENV{RRDCACHED_ADDRESS};
+        quickcheck('rrdcached',0,$ENV{RRDCACHED_ADDRESS},'Environment variable RRDCACHED_ADDRESS','n/a',\%mrtgrules);
+	}
+	if( exists $cfg->{ rrdcached } ) {
+        warn ("WARNING: You are running with RRDCached enabled (".$cfg->{ rrdcached }.").  This will disable all Threshold checking, since RRDCached does not support updatev and an update/fetch will cancel out the caching benefits.\n");
+        if( $cfg->{ rrdcached } !~ /^unix:/ ) {
+            warn("WARNING: You are running RRDCached in TCP mode.  This means that it will use its own Base Directory instead of WorkDir for storing the RRD files.  Also, changes to MaxBytes and DS Type will not be actioned after the RRD file has been created.\n");
+        }
+	}
+
 }
 
 # quick checks
@@ -644,15 +676,15 @@ sub quickcheck ($$$$$$) {
             return 1;
         } else {
             if ($second) {
-                die "ERROR: CFG Error in \"$first\[$second\]\", line $line: ".
+                die "ERROR: CFG Error in \"$first\[$second\]\", file $file line $line: ".
                   &{$rules->{$first.$braces}[1]}($arg)."\n\n"; 
             } else {
-                die "ERROR: CFG Error in \"$first\", line $line: ".
+                die "ERROR: CFG Error in \"$first\", file $file line $line: ".
                   &{$rules->{$first.$braces}[1]}($arg)."\n\n"; 
             } 
         }
     }
-    die "ERROR: CFG Error Unknown Option \"$first\" on line $line or above.\n".
+    die "ERROR: CFG Error Unknown Option \"$first\" in file $file on line $line or above.\n".
       "           Check doc/reference.txt for Help\n\n";
 }
 
@@ -763,16 +795,16 @@ sub cfgcheck ($$$$;$) {
     if(defined $$cfg{libadd}){
         ensureSL(\$$cfg{libadd});
         debug('eval',"libadd $$cfg{libadd}\n");
-	local $SIG{__DIE__};
+    	local $SIG{__DIE__};
         eval "use lib qw( $$cfg{libadd} )";
-	my @match;
-	foreach my $dir (@INC){
-		push @match, $dir if -f "$dir/RRDs.pm";
-	}
-	warn "WARN: found several copies of RRDs.pm in your path: ".
-        (join ", ", @match)." I will be using $match[0]. This could ".
-	"be a problem if this is an old copy and you think I would be using a newer one!\n"
-		if $#match > 0;
+    	my @match;
+	    foreach my $dir (@INC){
+		    push @match, $dir if -f "$dir/RRDs.pm";
+    	}
+	    warn "WARN: found several copies of RRDs.pm in your path: ".
+            (join ", ", @match)." I will be using $match[0]. This could ".
+        	"be a problem if this is an old copy and you think I would be using a newer one!\n"
+		    if $#match > 0;
     }
     $$cfg{logformat} = 'rateup' unless defined $$cfg{logformat};
 
@@ -836,6 +868,7 @@ sub cfgcheck ($$$$;$) {
         die "ERROR: File $cfg->{ conversioncode } conversion code evaluation failed\n$@\n"
             unless eval $code;
     }
+
     my $thresh_error;
 
     foreach $rou (@$routers) {
